// SPDX-License-Identifier: MIT
pragma solidity ^0.8.0;

contract SecureVotingSystem {
    struct Candidate {
        uint id;
        string name;
        uint voteCount;
    }

    struct Voter {
        bool authorized;
        bool voted;
        uint vote;
    }

    address public admin;
    uint public totalVotes;
    uint public startTime;
    uint public endTime;
    uint public candidateCount;

    mapping(address => Voter) public voters;
    mapping(uint => Candidate) public candidates;
    
    // Events
    event ElectionStarted(uint startTime, uint endTime);
    event VoterAuthorized(address voter);
    event VoteCast(address voter, uint candidateId);
    event ElectionEnded();

    modifier onlyAdmin() {
        require(msg.sender == admin, "Only admin can perform this action");
        _;
    }

    modifier onlyDuringElection() {
        require(block.timestamp >= startTime && block.timestamp <= endTime, "Voting is not active");
        _;
    }

    modifier onlyAfterElection() {
        require(block.timestamp > endTime, "Election is still active");
        _;
    }

    constructor() {
        admin = msg.sender;  // Set the contract deployer as the admin
    }

    // Set the election time period (start and end time)
    function setElectionTime(uint _startTime, uint _endTime) public onlyAdmin {
        require(_startTime < _endTime, "Start time must be before end time");
        startTime = _startTime;
        endTime = _endTime;
        emit ElectionStarted(_startTime, _endTime);
    }

    // Add candidates to the election
    function addCandidate(string memory _name) public onlyAdmin {
        candidateCount++;
        candidates[candidateCount] = Candidate(candidateCount, _name, 0);
    }

    // Authorize voters with multi-factor authentication (MFA) (simplified for demonstration)
    function authorizeVoter(address _voter) public onlyAdmin {
        voters[_voter].authorized = true;
        emit VoterAuthorized(_voter);
    }

    // Voting logic
    function vote(uint _candidateId) public onlyDuringElection {
        require(voters[msg.sender].authorized, "You are not authorized to vote");
        require(!voters[msg.sender].voted, "You have already voted");
        require(_candidateId > 0 && _candidateId <= candidateCount, "Invalid candidate");

        voters[msg.sender].voted = true;
        voters[msg.sender].vote = _candidateId;
        
        // Simulate vote encryption (in reality, you would need to implement encryption)
        bytes32 encryptedVote = keccak256(abi.encodePacked(msg.sender, _candidateId));
        
        candidates[_candidateId].voteCount++;
        totalVotes++;

        emit VoteCast(msg.sender, _candidateId);
    }

    // End the election and tally results
    function endElection() public onlyAdmin onlyAfterElection {
        emit ElectionEnded();
    }

    // Get detailed election results
    function getResult() public view onlyAfterElection returns (string memory winnerName, uint winnerVoteCount, uint totalVotesCast) {
        uint winningVoteCount = 0;
        uint winningCandidateId;

        for (uint i = 1; i <= candidateCount; i++) {
            if (candidates[i].voteCount > winningVoteCount) {
                winningVoteCount = candidates[i].voteCount;
                winningCandidateId = i;
            }
        }

        winnerName = candidates[winningCandidateId].name;
        winnerVoteCount = candidates[winningCandidateId].voteCount;
        totalVotesCast = totalVotes;
    }

    // Get the full vote tally for all candidates (for detailed reporting)
    function getAllCandidatesResults() public view onlyAfterElection returns (string[] memory candidateNames, uint[] memory voteCounts) {
        string[] memory names = new string[](candidateCount);
        uint[] memory votes = new uint[](candidateCount);

        for (uint i = 1; i <= candidateCount; i++) {
            names[i - 1] = candidates[i].name;
            votes[i - 1] = candidates[i].voteCount;
        }

        return (names, votes);
    }
}
